@@ -1,3 +1,5 @@
+#![allow(clippy::upper_case_acronyms)]
+
 use crate::ast::*;
 
 use lazy_static::lazy_static;
@@ -45,6 +47,10 @@ pub enum ParserError {
     MissingFunctionBody,
     #[error("Expected expression as parameter value, but got `{0}`")]
     ExpectedParamExpression(String),
+    #[error("Plot is missing a function name, but got nothing")]
+    PlotMissingFunction,
+    #[error("Expected function name, but got {0}")]
+    PlotUnexpectedSymbol(String),
 }
 
 #[derive(Parser)]
@@ -192,7 +198,7 @@ fn parse_function(function: Pairs<Rule>) -> Result<Statement, ParserError> {
             let body = parse_operand(p.into_inner())?;
             return Ok(Statement::Function {
                 name,
-                fun: Function { args, body },
+                fun: Function::Custom(CustomFunction { args, body }),
             });
         }
     }
@@ -200,6 +206,17 @@ fn parse_function(function: Pairs<Rule>) -> Result<Statement, ParserError> {
     Err(ParserError::MissingFunctionBody)
 }
 
+fn parse_plot(plot: Pairs<Rule>) -> Result<Statement, ParserError> {
+    let mut it = plot;
+    let fun = it.next().ok_or(ParserError::PlotMissingFunction)?;
+    match fun.as_rule() {
+        Rule::symbol => Ok(Statement::Plot {
+            name: fun.as_str().to_string(),
+        }),
+        _ => Err(ParserError::PlotUnexpectedSymbol(fun.as_str().to_string())),
+    }
+}
+
 fn parse_statement(statements: Pairs<Rule>) -> Result<Statement, ParserError> {
     let mut it = statements;
     let statement = it.next().ok_or(ParserError::EmptyStatement)?;
@@ -210,6 +227,7 @@ fn parse_statement(statements: Pairs<Rule>) -> Result<Statement, ParserError> {
         }),
         Rule::solvefor => parse_solve_for(statement.into_inner()),
         Rule::function => parse_function(statement.into_inner()),
+        Rule::plot => parse_plot(statement.into_inner()),
         r => Err(ParserError::InvalidStatement(format!(
             "Unexpected rule: {:?}",
             r
@@ -324,10 +342,10 @@ mod tests {
 
     #[test]
     fn parse_fun_no_args() {
-        let fun = Function {
+        let fun = Function::Custom(CustomFunction {
             args: Vec::new(),
             body: Operand::Number(12.0),
-        };
+        });
         let statement = Statement::Function {
             name: "ghs".to_string(),
             fun,
@@ -337,7 +355,7 @@ mod tests {
 
     #[test]
     fn parse_fun_x() {
-        let fun = Function {
+        let fun = Function::Custom(CustomFunction {
             args: vec!["x".to_string()],
             body: {
                 let lhs = Operand::Number(1.0);
@@ -345,7 +363,7 @@ mod tests {
                 let op = Operation::Add;
                 Operand::Term(Box::new(Term { lhs, rhs, op }))
             },
-        };
+        });
         let statement = Statement::Function {
             name: "f".to_string(),
             fun,
@@ -385,4 +403,12 @@ mod tests {
         let stat = Statement::Expression { op };
         assert_eq!(Ok(stat), parse("fun(42)"));
     }
+
+    #[test]
+    fn parse_plot() {
+        let stat = Statement::Plot {
+            name: "fun".to_string(),
+        };
+        assert_eq!(Ok(stat), parse("plot fun"));
+    }
 }
