@@ -163,6 +163,7 @@ mod tests {
     }
     use self::helpers::parse_expression;
     use super::*;
+    use crate::ast::CustomFunction;
     use crate::calc::TopLevelEnv;
     use crate::parse;
 
@@ -193,7 +194,7 @@ mod tests {
     #[test]
     fn normalize_operand_symbol_y() {
         let mut env = TopLevelEnv::default();
-        env.put("y".to_string(), 12.0);
+        env.put("y".to_string(), 12.0).unwrap();
         let act = normalize(&parse_expression("y"), "x", &env);
         assert_eq!(Ok(NormForm { a1: 0.0, a0: 12.0 }), act);
     }
@@ -322,14 +323,14 @@ mod tests {
         let mut env = TopLevelEnv::default();
         env.put_fun(
             "add".to_string(),
-            Function {
+            Function::Custom(CustomFunction {
                 args: vec!["x".to_string(), "y".to_string()],
                 body: Operand::Term(Box::new(Term {
                     lhs: Operand::Symbol("x".to_string()),
                     rhs: Operand::Symbol("y".to_string()),
                     op: Operation::Add,
                 })),
-            },
+            }),
         );
         assert!(if let Statement::SolveFor { lhs, rhs, sym } =
             parse("solve 2 * x + add(5, 12) = 22 - 6 * x + 7 for x").unwrap()
